-- shape charge
--
-- description
-- adds an shaped charge with blows up an area of 3x3x5 blocks, but for now preservers most usefull blocks
--
-- shortcommings
-- - detonates instantly on hit --> cannot be reclaimed if placed wrong
-- - needs a proper texture
--
--
-- Licence:
--   Code: GPL
--   Texture: GPL (shapedcharge_shape_charge.png by Joe McCay)

--minetest.register_on_placenode(function(pos, newnode, placer)
--        if newnode.name == "shaped_charge:shaped_charge" then
--            -- minetest.env:add_node(pos, {name="shaped_charge:shaped_charge", param2=param2})
--            -- place_shaped_charge(pos, newnode, placer)
--        end
--    end
--)

-- From Screwdriver mod
local function get_node_face(pointed_thing)
	local ax, ay, az = pointed_thing.above.x, pointed_thing.above.y, pointed_thing.above.z
	local ux, uy, uz = pointed_thing.under.x, pointed_thing.under.y, pointed_thing.under.z
	if     ay > uy then return 0 -- Top
	elseif az > uz then return 1 -- Z+ side
	elseif az < uz then return 2 -- Z- side
	elseif ax > ux then return 3 -- X+ side
	elseif ax < ux then return 4 -- X- side
	elseif ay < uy then return 5 -- Bottom
	else
		error("pointed_thing.above and under are the same!")
	end
end

minetest.register_on_dignode(function(pos, oldnode, digger)
        if oldnode.name == "shapedcharge:shaped_charge" then
            remove_shaped_charge(pos, oldnode, digger, true)
        end
    end
)

minetest.register_on_punchnode(function(pos, oldnode, digger)
        if oldnode.name == "shapedcharge:shaped_charge" then
            trigger_shaped_charge(pos, oldnode, digger, false)
        end
    end
)

remove_shaped_charge = function(pos, oldnode, digger, completely)
    print "shapedcharge removed"
    return true
end

trigger_shaped_charge = function(pos, oldnode, digger, completely, pointerthing)
    local destroyable_table = {"air", "default:sand", "default:dirt", "default:dirt_with_grass", "default:grass", "default:cobble", "default:mossycobble", "default:gravel", "default:stone"}
	
	-- From Screwdriver Mod
	local opposite_faces = {
		[0] = 5,
		[1] = 2,
		[2] = 1,
		[3] = 4,
		[4] = 3,
		[5] = 0,
	}
    
    print "shapedcharge triggered"
    local n = minetest.env:get_node(pos)
    --direction = n.param2
	--local direction = digger:get_look_dir()
	local direction = opposite_faces[get_node_face(pointerthing)]
	
    --for key, item  in pairs(n) do
    --    print("key:",  key)
    --    print("value", item)
    --end
    
    -- direction : value of param2
    -- x-pos     :  1
    -- x-neg     :  2
    -- up   (-y) :  4
    -- down (+y) :  8
    -- z-pos     : 16
    -- z-neg     : 32

    local d_a_min = -1
    local d_a_max =  1
    local d_b_min = -1
    local d_b_max =  1
    local d_c_min =  1
    local d_c_max =  5
	local p
	local destroyable
    
    --print("base pos : ", pos.x, " ", pos.y, " ", pos.z)
	--print("direction ", direction)

    --the actual destruction loop
    for d_a = d_a_min, d_a_max do
        for d_b = d_b_min, d_b_max do
            for d_c = d_c_min, d_c_max do
                -- first map abc to xyz based on heading of shaped charge
                if direction == 3 then        -- x-pos
                    p = {x=pos.x+d_c, y=pos.y+d_b, z=pos.z+d_a}
                elseif direction == 4 then    -- x-neg
                    p = {x=pos.x-d_c, y=pos.y+d_b, z=pos.z+d_a}
                elseif direction == 0 then    -- y-pos
                    p = {x=pos.x+d_a, y=pos.y+d_c, z=pos.z+d_b}
                elseif direction == 5 then    -- y-neg
                    p = {x=pos.x+d_a, y=pos.y-d_c, z=pos.z+d_b}
                elseif direction == 1 then   -- z-pos
                    p = {x=pos.x+d_a, y=pos.y+d_b, z=pos.z+d_c}
                elseif direction == 2 then   -- z-neg
                    p = {x=pos.x+d_a, y=pos.y+d_b, z=pos.z-d_c}
                else
                  print("shapedcharge: warning unexpected direction")
                  return false
                end

                -- some precautions so we don't blow up usefull stuff
                n = minetest.env:get_node(p)
                --print("checking: ", p.x, " ", p.y, " ", p.z)
                destroyable = false
                
                -- for now don't destroy anything with has param1 or param2 set
                if n.name ~= "air" and (n.param1 ~= nil and n.param1 ~= 0) or (n.param2 ~= nil and n.param2 ~= 0) then
                  --print("checking: ", p.x, " ", p.y, " ", p.z)
                  --print("Node:",n.name,"param1:", n.param1, "param2:", n.param2)
                  --break
                end

                -- don't blow up ore
                if n.name == "default:stone" and n.param1 ~= 0 then
                  -- stone contains ore --> don't destroy
				  --destroyable = false
                  --break
                end

                -- only blow up items which are included in the destroyable_table
                for _,item in ipairs(destroyable_table) do
                    if n.name == item then
                        destroyable = true
                        break
                    end
                end

                if destroyable==true then
                    -- finally blow it up
                    minetest.env:remove_node(p)
                else
                    -- abort if something is hit which can/shall not be destroyed
                    --print("checking: ", p.x, " ", p.y, " ", p.z)
                    --print("row aborted, because a", n.name, "Node was hit")
                    --break
                end
            end
        end
    end

    -- remove the charge itself
    minetest.env:remove_node(pos)

    return true
end

minetest.register_craft({
    output = "shapedcharge:shaped_charge 1",
    --recipe = {
    --    {''           , 'node "wood"'            , ''           },
    --    {'node "wood"', 'node "experimental:tnt"', 'node "wood"'},
    --    {''           , 'node "wood"'            , ''           },
    --}
    recipe = {
        {""                   , "default:steel_ingot", ""             },
        {"default:wood"       , "tnt:tnt"            , "default:wood" },
        {"default:wood"       , "default:wood"       , "default:wood" },
    }
})

minetest.register_node("shapedcharge:shaped_charge", {
    description = "Shaped charge",
	drawtype = "signlike",
    tiles = {"shapedcharge_shape_charge.png"},
    inventory_image = "shapedcharge_shape_charge.png",
	wield_image = "shapedcharge_shape_charge.png",
    light_propagates = true,
    paramtype = "light",
	paramtype2 = "wallmounted",
    is_ground_content = true,
    legacy_wallmounted = true,
    walkable = false,
    climbable = false,
    selection_box = {
        type = "wallmounted",
	--wall_top = = <default>
	--wall_bottom = = <default>
	--wall_side = = <default>
    },
	groups = {choppy=2,dig_immediate=2,attached_node=1},
	
	--on_punch = function(pos, oldnode, digger)
	on_punch = function(pos, oldnode, digger, pointerthing)
        if oldnode.name == "shapedcharge:shaped_charge" then
            trigger_shaped_charge(pos, oldnode, digger, false, pointerthing)
        end
    end,
	
    furnace_burntime = 5,
    material = {
	diggablity = "normal",
	cuttability = 1.5,
    },
})

--helper

function table.contains(table, element)
  for _, value in pairs(table) do
    if value == element then
      return true
    end
  end
  return false
end

